# -*- coding: utf-8 -*-
"""
Adapters to help render :mod:`crabpy.gateway.capakey` objects to json.

.. versionadded:: 0.1.0
"""
from crabpy.gateway import capakey

from pyramid.renderers import JSON

json_list_renderer = JSON()
json_item_renderer = JSON()


def list_gemeente_adapter(obj, request):
    """
    Adapter for rendering a list of
    :class: `crabpy.gateway.capakey.Gemeente` to json.
    """
    return {
        'id': obj.id,
        'naam': obj.naam
    }


def list_afdeling_adapter(obj, request):
    """
    Adapter for rendering a list of
    :class: `crabpy.gateway.capakey.Afdeling` to json.
    """
    return {
        'id': obj.id,
        'naam': obj.naam,
        'gemeente': obj.gemeente
    }


def list_sectie_adapter(obj, request):
    """
    Adapter for rendering a list of
    :class: `crabpy.gateway.capakey.Sectie` to json.
    """
    return {
        'id': obj.id,
        'afdeling': obj.afdeling
    }


def list_perceel_adapter(obj, request):
    """
    Adapter for rendering a list of
    :class: `crabpy.gateway.capakey.Perceel` to json.
    """
    return {
        'id': obj.id,
        'sectie': obj.sectie,
        'capakey': obj.capakey,
        'percid': obj.percid
    }


json_list_renderer.add_adapter(capakey.Gemeente, list_gemeente_adapter)
json_list_renderer.add_adapter(capakey.Afdeling, list_afdeling_adapter)
json_list_renderer.add_adapter(capakey.Sectie, list_sectie_adapter)
json_list_renderer.add_adapter(capakey.Perceel, list_perceel_adapter)


def item_gemeente_adapter(obj, request):
    """
    Adapter for rendering an object of
    :class: `crabpy.gateway.capakey.Gemeente` to json.
    """
    return {
        'id': obj.id,
        'naam': obj.naam,
        'centroid': obj.centroid,
        'bounding_box': obj.bounding_box
    }


def item_afdeling_adapter(obj, request):
    """
    Adapter for rendering an object of
    :class: `crabpy.gateway.capakey.Afdeling` to json.
    """
    return {
        'id': obj.id,
        'naam': obj.naam,
        'gemeente': {
            'id': obj.gemeente.id,
            'naam': obj.gemeente.naam
        },
        'centroid': obj.centroid,
        'bounding_box': obj.bounding_box
    }


def item_sectie_adapter(obj, request):
    """
    Adapter for rendering an object of
    :class: `crabpy.gateway.capakey.Sectie` to json.
    """
    return {
        'id': obj.id,
        'afdeling': {
            'id': obj.afdeling.id,
            'naam': obj.afdeling.naam,
            'gemeente': {
                'id': obj.afdeling.gemeente.id,
                'naam': obj.afdeling.gemeente.naam
            },
        },
        'centroid': obj.centroid,
        'bounding_box': obj.bounding_box
    }


def item_perceel_adapter(obj, request):
    """
    Adapter for rendering an object of
    :class: `crabpy.gateway.capakey.Perceel` to json.
    """
    return {
        'id': obj.id,
        'sectie': {
            'id': obj.sectie.id,
            'afdeling': {
                'id': obj.sectie.afdeling.id,
                'naam': obj.sectie.afdeling.naam,
                'gemeente': {
                    'id': obj.sectie.afdeling.gemeente.id,
                    'naam': obj.sectie.afdeling.gemeente.naam
                },
            },
        },
        'capakey': obj.capakey,
        'percid': obj.percid,
        'centroid': obj.centroid,
        'bounding_box': obj.bounding_box,
        'shape': obj.shape
    }


json_item_renderer.add_adapter(capakey.Gemeente, item_gemeente_adapter)
json_item_renderer.add_adapter(capakey.Afdeling, item_afdeling_adapter)
json_item_renderer.add_adapter(capakey.Sectie, item_sectie_adapter)
json_item_renderer.add_adapter(capakey.Perceel, item_perceel_adapter)



# -*- coding: utf-8 -*-
"""
Routes for the CAPAKEY views.

.. versionadded:: 0.1.0
"""
import crabpy_pyramid


def includeme(config):
    crabpy_pyramid.add_route(config,
                             'list_gemeenten',
                             '/capakey/gemeenten')
    crabpy_pyramid.add_route(config,
                             'get_gemeente',
                             '/capakey/gemeenten/{gemeente_id}')
    crabpy_pyramid.add_route(config,
                             'list_kadastrale_afdelingen_by_gemeente',
                             '/capakey/gemeenten/{gemeente_id}/afdelingen')
    crabpy_pyramid.add_route(config,
                             'list_kadastrale_afdelingen',
                             '/capakey/afdelingen')
    crabpy_pyramid.add_route(config,
                             'get_kadastrale_afdeling_by_id',
                             '/capakey/afdelingen/{afdeling_id}')
    crabpy_pyramid.add_route(config,
                             'list_secties_by_afdeling',
                             '/capakey/afdelingen/{afdeling_id}/secties')
    crabpy_pyramid.add_route(
        config,
        'get_sectie_by_id_and_afdeling',
        '/capakey/afdelingen/{afdeling_id}/secties/{sectie_id}')
    crabpy_pyramid.add_route(
        config,
        'list_percelen_by_sectie',
        '/capakey/afdelingen/{afdeling_id}/secties/{sectie_id}/percelen')
    crabpy_pyramid.add_route(
        config,
        'get_perceel_by_sectie_and_id',
        '/capakey/afdelingen/{afdeling_id}/secties/{sectie_id}'
        '/percelen/{perceel_id1}/{perceel_id2}')
    crabpy_pyramid.add_route(config,
                             'get_perceel_by_capakey',
                             '/capakey/percelen/{capakey1}/{capakey2}')
    crabpy_pyramid.add_route(config,
                             'get_perceel_by_percid',
                             '/capakey/percelen/{percid}')



import crabpy_pyramid


def includeme(config):
    crabpy_pyramid.add_route(
        config, "adressenregister_list_gewesten", "/adressenregister/gewesten"
    )
    crabpy_pyramid.add_route(
        config,
        "adressenregister_get_gewest_by_niscode",
        "/adressenregister/gewesten/{gewest_niscode}",
    )

    crabpy_pyramid.add_route(
        config,
        "adressenregister_list_provincies",
        "/adressenregister/gewesten/{gewest_niscode}/provincies",
    )
    crabpy_pyramid.add_route(
        config,
        "adressenregister_get_provincie",
        "/adressenregister/provincies/{provincie_niscode}",
    )
    crabpy_pyramid.add_route(
        config,
        "adressenregister_list_deelgemeenten",
        "/adressenregister/gewesten/{gewest_niscode}/deelgemeenten",
    )
    crabpy_pyramid.add_route(
        config,
        "adressenregister_list_deelgemeenten_by_gemeente",
        "/adressenregister/gemeenten/{niscode}/deelgemeenten",
    )
    crabpy_pyramid.add_route(
        config,
        "adressenregister_get_deelgemeente_by_id",
        "/adressenregister/deelgemeenten/{deelgemeente_niscode}",
    )

    crabpy_pyramid.add_route(
        config,
        "adressenregister_list_gemeenten_by_provincie",
        "/adressenregister/provincies/{provincie_niscode}/gemeenten",
    )
    crabpy_pyramid.add_route(
        config,
        "list_gemeenten_adressenregister",
        "/adressenregister/gewesten/{gewest_niscode}/gemeenten",
    )

    crabpy_pyramid.add_route(
        config, "get_gemeente_adressenregister", "/adressenregister/gemeenten/{niscode}"
    )

    crabpy_pyramid.add_route(
        config,
        "adressenregister_list_straten",
        "/adressenregister/gemeenten/{niscode}/straten",
    )

    crabpy_pyramid.add_route(
        config,
        "adressenregister_get_straat_by_id",
        "/adressenregister/straten/{straat_id}",
    )

    crabpy_pyramid.add_route(
        config,
        "adressenregister_list_adressen",
        "/adressenregister/straten/{straat_id}/adressen",
    )

    crabpy_pyramid.add_route(
        config,
        "adressenregister_get_adres_by_straat_and_huisnummer",
        "/adressenregister/straten/{straat_id}/huisnummers/{huisnummer}",
    )

    crabpy_pyramid.add_route(
        config,
        "adressenregister_get_adres_by_straat_and_huisnummer_and_busnummer",
        "/adressenregister/straten/{straat_id}/huisnummers/{huisnummer}/busnummers/{busnummer}",
    )

    crabpy_pyramid.add_route(
        config,
        "adressenregister_get_adres_by_id",
        "/adressenregister/adressen/{adres_id}",
    )

    crabpy_pyramid.add_route(
        config,
        "adressenregister_list_percelen_by_adres",
        "/adressenregister/adressen/{adres_id}/percelen",
    )
    crabpy_pyramid.add_route(
        config,
        "adressenregister_get_perceel_by_id",
        "/adressenregister/percelen/{perceel_id}",
    )
    crabpy_pyramid.add_route(
        config,
        "adressenregister_get_perceel_by_id_parts",
        "/adressenregister/percelen/{perceel_id_part1}/{perceel_id_part2}",
    )
    crabpy_pyramid.add_route(
        config,
        "adressenregister_list_postinfo_by_gemeente",
        "/adressenregister/gemeenten/{gemeente_naam_niscode}/postinfo",
    )
    crabpy_pyramid.add_route(
        config,
        "adressenregister_get_postinfo_by_postcode",
        "/adressenregister/postinfo/{postcode}",
    )

    crabpy_pyramid.add_route(
        config, "adressenregister_list_landen", "/adressenregister/landen"
    )
    crabpy_pyramid.add_route(
        config, "adressenregister_get_land_by_id", "/adressenregister/landen/{land_id}"
    )



# -*- coding: utf-8 -*-
"""
Utility functions to help with range handling.

.. versionadded:: 0.1.0
"""

import re

MAX_NUMBER_ITEMS = 5000


def parse_range_header(range):
    """
    Parse a range header as used by the dojo Json Rest store.

    :param str range: The content of the range header to be parsed. 
        eg. `items=0-9`
    :returns: A dict with keys start, finish and number or `False` if the
        range is invalid.
    """
    match = re.match('^items=([0-9]+)-([0-9]+)$', range)

    if match:
        start = int(match.group(1))
        finish = int(match.group(2))

        if finish < start:
            finish = start
        return {
            'start': start,
            'finish': finish,
            'count': finish - start + 1
        }
    else:
        return False


def range_return(request, items):
    """
    Determine what range of objects to return.

    Will check fot both `Range` and `X-Range` headers in the request and
    set both `Content-Range` and 'X-Content-Range' headers.

    :rtype: list
    """
    if ('Range' in request.headers):
        range = parse_range_header(request.headers['Range'])
    elif 'X-Range' in request.headers:
        range = parse_range_header(request.headers['X-Range'])
    else:
        range = {
            'start': 0,
            'finish': MAX_NUMBER_ITEMS - 1,
            'count': MAX_NUMBER_ITEMS
        }
    filtered = items[range['start']:range['finish'] + 1]
    if len(filtered) < range['count']:
        # Something was stripped, deal with it
        range['count'] = len(filtered)
        range['finish'] = range['start'] + range['count'] - 1
    if range['finish'] - range['start'] + 1 >= MAX_NUMBER_ITEMS:
        range['finish'] = range['start'] + MAX_NUMBER_ITEMS - 1
        filtered = items[range['start']:range['finish'] + 1]

    request.response.headers['Content-Range'] = 'items %d-%d/%d' % (range['start'], range['finish'], len(items))
    request.response.headers['X-Content-Range'] = request.response.headers['Content-Range']
    return filtered


def set_http_caching(request, gateway='crab', region='permanent'):
    """
    Set an HTTP Cache Control header on a request.

    :param pyramid.request.Request request: Request to set headers on.
    :param str gateway: What gateway are we caching for? Defaults to `crab`.
    :param str region: What caching region to use? Defaults to `permanent`.
    :rtype: pyramid.request.Request
    """
    crabpy_exp = request.registry.settings.get('crabpy.%s.cache_config.%s.expiration_time' % (gateway, region), None)
    if crabpy_exp is None:
        return request
    ctime = int(int(crabpy_exp) * 1.05)
    request.response.cache_expires(ctime, public=True)
    return request



# -*- coding: utf-8 -*-
"""
Views for CRAB services

.. versionadded:: 0.1.0
"""
import logging

import pycountry
from crabpy.gateway.exception import GatewayResourceNotFoundException
from pyramid.httpexceptions import HTTPNotFound
from pyramid.view import view_config

from crabpy_pyramid.utils import range_return
from crabpy_pyramid.utils import set_http_caching

log = logging.getLogger(__name__)


@view_config(
    route_name='list_gewesten',
    renderer='crab_listjson', accept='application/json'
)
def list_gewesten(request):
    request = set_http_caching(request, 'crab', 'permanent')
    Gateway = request.crab_gateway()
    sort = request.params.get('sort', 1)
    gewesten = Gateway.list_gewesten(sort)
    return range_return(request, gewesten)


@view_config(
    route_name='get_gewest_by_id',
    renderer='crab_itemjson', accept='application/json'
)
def get_gewest_by_id(request):
    request = set_http_caching(request, 'crab', 'permanent')
    Gateway = request.crab_gateway()
    gewest_id = int(request.matchdict.get('gewest_id'))
    try:
        return Gateway.get_gewest_by_id(gewest_id)
    except GatewayResourceNotFoundException:
        return HTTPNotFound()


@view_config(
    route_name='list_provincies',
    renderer='crab_listjson', accept='application/json'
)
def list_provincies(request):
    request = set_http_caching(request, 'crab', 'permanent')
    Gateway = request.crab_gateway()
    gewest_id = int(request.matchdict.get('gewest_id'))
    provincies = Gateway.list_provincies(gewest_id)
    return range_return(request, provincies)


@view_config(
    route_name='get_provincie',
    renderer='crab_itemjson', accept='application/json'
)
def get_provincie(request):
    request = set_http_caching(request, 'crab', 'permanent')
    Gateway = request.crab_gateway()
    provincie_id = int(request.matchdict.get('provincie_id'))
    try:
        return Gateway.get_provincie_by_id(provincie_id)
    except GatewayResourceNotFoundException:
        return HTTPNotFound()


@view_config(
    route_name='list_gemeenten_by_provincie',
    renderer='crab_listjson', accept='application/json'
)
def list_gemeenten_by_provincie(request):
    request = set_http_caching(request, 'crab', 'long')
    Gateway = request.crab_gateway()
    provincie_id = int(request.matchdict.get('provincie_id'))
    gemeenten = Gateway.list_gemeenten_by_provincie(provincie_id)
    return range_return(request, gemeenten)


@view_config(
    route_name='list_gemeenten_crab',
    renderer='crab_listjson', accept='application/json'
)
def list_gemeenten_crab(request):
    request = set_http_caching(request, 'crab', 'permanent')
    Gateway = request.crab_gateway()
    sort = request.params.get('sort', 'niscode')
    sort_map = {'id': 1, 'naam': 2, 'niscode': 6}
    sort = sort_map.get(sort, 6)
    gewest_id = request.matchdict.get('gewest_id')
    gemeenten = Gateway.list_gemeenten(gewest_id, sort)
    return range_return(request, gemeenten)


@view_config(
    route_name='get_gemeente_crab',
    renderer='crab_itemjson', accept='application/json'
)
def get_gemeente_crab(request):
    request = set_http_caching(request, 'crab', 'long')
    Gateway = request.crab_gateway()
    gemeente_id = request.matchdict.get('gemeente_id')
    if len(gemeente_id) == 5:
        try:
            return Gateway.get_gemeente_by_niscode(gemeente_id)
        except GatewayResourceNotFoundException:
            return HTTPNotFound()
    else:
        try:
            return Gateway.get_gemeente_by_id(gemeente_id)
        except GatewayResourceNotFoundException:
            return HTTPNotFound()


@view_config(
    route_name='list_deelgemeenten',
    renderer='crab_listjson', accept='application/json'
)
def list_deelgemeenten(request):
    Gateway = request.crab_gateway()
    gewest_id = int(request.matchdict.get('gewest_id'))
    if gewest_id != 2:
        return HTTPNotFound()
    deelgemeenten = Gateway.list_deelgemeenten(gewest_id)
    return range_return(request, deelgemeenten)


@view_config(
    route_name='list_deelgemeenten_by_gemeente',
    renderer='crab_listjson', accept='application/json'
)
def list_deelgemeenten_by_gemeente(request):
    request = set_http_caching(request, 'crab', 'permanent')
    Gateway = request.crab_gateway()
    gemeente_id = request.matchdict.get('gemeente_id')
    if len(gemeente_id) == 5:
        try:
            gemeente = Gateway.get_gemeente_by_niscode(gemeente_id)
        except GatewayResourceNotFoundException:
            return HTTPNotFound()
    else:
        try:
            gemeente = Gateway.get_gemeente_by_id(gemeente_id)
        except GatewayResourceNotFoundException:
            return HTTPNotFound()
    deelgemeenten = Gateway.list_deelgemeenten_by_gemeente(gemeente)
    return range_return(request, deelgemeenten)


@view_config(
    route_name='get_deelgemeente_by_id',
    renderer='crab_itemjson', accept='application/json'
)
def get_deelgemeente_by_id(request):
    request = set_http_caching(request, 'crab', 'permanent')
    Gateway = request.crab_gateway()
    deelgemeente_id = request.matchdict.get('deelgemeente_id')
    try:
        return Gateway.get_deelgemeente_by_id(deelgemeente_id)
    except GatewayResourceNotFoundException:
        return HTTPNotFound()


@view_config(
    route_name='list_straten',
    renderer='crab_listjson', accept='application/json'
)
def list_straten(request):
    request = set_http_caching(request, 'crab', 'long')
    Gateway = request.crab_gateway()
    gemeente_id = request.matchdict.get('gemeente_id')
    if len(gemeente_id) == 5:
        gemeente_id = Gateway.get_gemeente_by_niscode(gemeente_id)
    straten = Gateway.list_straten(gemeente_id)
    return range_return(request, straten)


@view_config(
    route_name='get_straat_by_id',
    renderer='crab_itemjson', accept='application/json'
)
def get_straat_by_id(request):
    request = set_http_caching(request, 'crab', 'long')
    Gateway = request.crab_gateway()
    straat_id = request.matchdict.get('straat_id')
    try:
        return Gateway.get_straat_by_id(straat_id)
    except GatewayResourceNotFoundException:
        return HTTPNotFound()


@view_config(
    route_name='list_huisnummers',
    renderer='crab_listjson', accept='application/json'
)
def list_huisnummers(request):
    request = set_http_caching(request, 'crab', 'short')
    Gateway = request.crab_gateway()
    straat_id = request.matchdict.get('straat_id')
    sort = request.params.get('sort', 1)
    huisnummers = Gateway.list_huisnummers_by_straat(straat_id, sort)
    return range_return(request, huisnummers)


@view_config(
    route_name='get_huisnummer_by_straat_and_label',
    renderer='crab_itemjson', accept='application/json'
)
def get_huisnummer_by_straat_and_label(request):
    request = set_http_caching(request, 'crab', 'short')
    Gateway = request.crab_gateway()
    straat_id = request.matchdict.get('straat_id')
    huisnummer = request.matchdict.get('huisnummer_label')
    try:
        return Gateway.get_huisnummer_by_nummer_and_straat(huisnummer, straat_id)
    except GatewayResourceNotFoundException:
        return HTTPNotFound()


@view_config(
    route_name='get_huisnummer_by_id',
    renderer='crab_itemjson', accept='application/json'
)
def get_huisnummer_by_id(request):
    request = set_http_caching(request, 'crab', 'short')
    Gateway = request.crab_gateway()
    huisnummer_id = request.matchdict.get('huisnummer_id')
    try:
        return Gateway.get_huisnummer_by_id(huisnummer_id)
    except GatewayResourceNotFoundException:
        return HTTPNotFound()


@view_config(
    route_name='list_percelen',
    renderer='crab_listjson', accept='application/json'
)
def list_percelen(request):
    request = set_http_caching(request, 'crab', 'short')
    Gateway = request.crab_gateway()
    huisnummer_id = request.matchdict.get('huisnummer_id')
    percelen = Gateway.list_percelen_by_huisnummer(huisnummer_id)
    return range_return(request, percelen)


@view_config(
    route_name='get_perceel_by_id',
    renderer='crab_itemjson', accept='application/json'
)
def get_perceel_by_id(request):
    request = set_http_caching(request, 'crab', 'short')
    Gateway = request.crab_gateway()
    perceel_id = request.matchdict.get('perceel_id1') + '/' + request.matchdict.get('perceel_id2')
    try:
        return Gateway.get_perceel_by_id(perceel_id)
    except GatewayResourceNotFoundException:
        return HTTPNotFound()


@view_config(
    route_name='list_huisnummers_by_perceel',
    renderer='crab_listjson', accept='application/json'
)
def list_huisnummers_by_perceel(request):
    request = set_http_caching(request, 'crab', 'short')
    Gateway = request.crab_gateway()
    perceel_id = request.matchdict.get('perceel_id1') + '/' + request.matchdict.get('perceel_id2')
    sort = request.params.get('sort', 1)
    try:
        perceel = Gateway.get_perceel_by_id(perceel_id)
        return Gateway.list_huisnummers_by_perceel(perceel, sort)
    except GatewayResourceNotFoundException:
        return HTTPNotFound()


@view_config(
    route_name='list_gebouwen',
    renderer='crab_listjson', accept='application/json'
)
def list_gebouwen(request):
    request = set_http_caching(request, 'crab', 'short')
    Gateway = request.crab_gateway()
    huisnummer_id = request.matchdict.get('huisnummer_id')
    gebouwen = Gateway.list_gebouwen_by_huisnummer(huisnummer_id)
    return range_return(request, gebouwen)


@view_config(
    route_name='get_gebouw_by_id',
    renderer='crab_itemjson', accept='application/json'
)
def get_gebouw_by_id(request):
    request = set_http_caching(request, 'crab', 'short')
    Gateway = request.crab_gateway()
    gebouw_id = request.matchdict.get('gebouw_id')
    try:
        return Gateway.get_gebouw_by_id(gebouw_id)
    except GatewayResourceNotFoundException:
        return HTTPNotFound()


@view_config(
    route_name='get_wegobject',
    renderer='crab_itemjson', accept='application/json'
)
def get_wegobject(request):
    request = set_http_caching(request, 'crab', 'short')
    Gateway = request.crab_gateway()
    wegobject_id = request.matchdict.get('wegobject_id')
    try:
        return Gateway.get_wegobject_by_id(wegobject_id)
    except GatewayResourceNotFoundException:
        return HTTPNotFound()


@view_config(
    route_name='list_subadressen',
    renderer='crab_listjson', accept='application/json'
)
def list_subadressen(request):
    request = set_http_caching(request, 'crab', 'short')
    Gateway = request.crab_gateway()
    huisnummer_id = request.matchdict.get('huisnummer_id')
    subadressen = Gateway.list_subadressen_by_huisnummer(huisnummer_id)
    return range_return(request, subadressen)


@view_config(
    route_name='get_subadres_by_id',
    renderer='crab_itemjson', accept='application/json'
)
def get_subadres_by_id(request):
    request = set_http_caching(request, 'crab', 'short')
    Gateway = request.crab_gateway()
    subadres_id = request.matchdict.get('subadres_id')
    try:
        return Gateway.get_subadres_by_id(subadres_id)
    except GatewayResourceNotFoundException:
        return HTTPNotFound()


@view_config(
    route_name='list_postkantons_by_gemeente',
    renderer='crab_listjson', accept='application/json'
)
def list_postkantons_by_gemeente(request):
    request = set_http_caching(request, 'crab', 'long')
    Gateway = request.crab_gateway()
    gemeente_id = request.matchdict.get('gemeente_id')
    postkantons = Gateway.list_postkantons_by_gemeente(gemeente_id)
    return range_return(request, postkantons)


@view_config(
    route_name='list_adresposities_by_huisnummer',
    renderer='crab_listjson', accept='application/json'
)
def list_adresposities_by_huisnummer(request):
    request = set_http_caching(request, 'crab', 'short')
    Gateway = request.crab_gateway()
    huisnummer_id = request.matchdict.get('huisnummer_id')
    adresposities = Gateway.list_adresposities_by_huisnummer(huisnummer_id)
    return range_return(request, adresposities)


@view_config(
    route_name='list_adresposities_by_subadres',
    renderer='crab_listjson', accept='application/json'
)
def list_adresposities_by_subadres(request):
    request = set_http_caching(request, 'crab', 'short')
    Gateway = request.crab_gateway()
    subadres_id = request.matchdict.get('subadres_id')
    adresposities = Gateway.list_adresposities_by_subadres(subadres_id)
    return range_return(request, adresposities)


@view_config(
    route_name='get_adrespositie_by_id',
    renderer='crab_itemjson', accept='application/json'
)
def get_adrespositie_by_id(request):
    request = set_http_caching(request, 'crab', 'short')
    Gateway = request.crab_gateway()
    adrespositie_id = request.matchdict.get('adrespositie_id')
    try:
        return Gateway.get_adrespositie_by_id(adrespositie_id)
    except GatewayResourceNotFoundException:
        return HTTPNotFound()


@view_config(
    route_name='list_landen',
    renderer='crab_listjson', accept='application/json'
)
def list_landen(request):
    request = set_http_caching(request, 'crab', 'permanent')
    return list(pycountry.countries)


@view_config(
    route_name='get_land_by_id',
    renderer='crab_itemjson', accept='application/json'
)
def get_land_by_id(request):
    request = set_http_caching(request, 'crab', 'permanent')
    land_id = request.matchdict.get('land_id')
    land = pycountry.countries.get(alpha_2=land_id)
    if land is None:
        return HTTPNotFound()
    return land


@view_config(
    route_name='get_postkanton_by_huisnummer',
    renderer='crab_itemjson', accept='application/json'
)
def get_postkanton_by_huisnummer(request):
    request = set_http_caching(request, 'crab', 'short')
    Gateway = request.crab_gateway()
    huisnummer_id = request.matchdict.get('huisnummer_id')
    try:
        return Gateway.get_postkanton_by_huisnummer(huisnummer_id)
    except GatewayResourceNotFoundException:
        return HTTPNotFound()



import logging
import sys

from crabpy.client import AdressenRegisterClientException
from pyramid.view import view_config

LOGGER = logging.getLogger(__name__)

@view_config(
    context=AdressenRegisterClientException,
    renderer="json",
    accept="application/json"
)
def internal_server_error(exception, request):
    LOGGER.exception(exception)
    original_exception = exception.__cause__
    request.response.status_int = 500
    errors = [str(original_exception)]

    return {
        "message": "Er ging iets fout in de vraag naar adressenregister API.",
        "Errors": errors

    }



import logging
import os
from  collections.abc import Sequence

from crabpy.client import AdressenRegisterClient
from crabpy.client import crab_factory
from crabpy.gateway.adressenregister import Gateway
from crabpy.gateway.capakey import CapakeyRestGateway
from crabpy.gateway.crab import CrabGateway
from pyramid.config import Configurator
from pyramid.settings import asbool
from zope.interface import Interface

from crabpy_pyramid.renderers.adressenregister import \
    json_item_renderer as adresreg_json_item_renderer
from crabpy_pyramid.renderers.adressenregister import \
    json_list_renderer as adresreg_json_list_renderer
from crabpy_pyramid.renderers.capakey import \
    json_item_renderer as capakey_json_item_renderer
from crabpy_pyramid.renderers.capakey import \
    json_list_renderer as capakey_json_list_renderer
from crabpy_pyramid.renderers.crab import json_item_renderer as crab_json_item_renderer
from crabpy_pyramid.renderers.crab import json_list_renderer as crab_json_list_renderer

log = logging.getLogger(__name__)
GENERATE_ETAG_ROUTE_NAMES = set()


class ICapakey(Interface):
    pass


class ICrab(Interface):
    pass


class IAdressenregister(Interface):
    pass

def _parse_settings(settings):
    defaults = {
        'capakey.include': False,
        'crab.include': True,
        'adressenregister.include': True,
        'adressenregister.base_url': 'https://api.basisregisters.vlaanderen.be',
        'adressenregister.api_key': None,
        'cache.file.root': '/tmp/dogpile_data',
    }
    args = defaults.copy()
    if 'adressenregister.api_key' not in settings:
        log.warning(
            "No adressenregister.api_key set in settings. "
            "The api might stop working after reaching the limit of x requests per day."
        )

    # booelean settings
    for short_key_name in ('capakey.include', 'crab.include', 'adressenregister.include'):
        key_name = "crabpy.%s" % short_key_name
        if key_name in settings:
            args[short_key_name] = asbool(settings.get(
                key_name, defaults.get(short_key_name)
            ))

    # string setting
    for short_key_name in (
        'proxy.http',
        'proxy.https',
        'cache.file.root',
        'adressenregister.base_url',
        'adressenregister.api_key'
    ):
        key_name = "crabpy.%s" % short_key_name
        if key_name in settings:
            args[short_key_name] = settings.get(key_name)

    # cache configuration
    for short_key_name in (
        'crab.cache_config',
        'capakey.cache_config',
        'adressenregister.cache_config'
    ):
        key_name = "crabpy.%s." % short_key_name
        cache_config = {}
        for skey in settings.keys():
            if skey.startswith(key_name):
                cache_config[skey[len(key_name):]] = settings.get(skey)
        if cache_config:
            args[short_key_name] = cache_config

    # crab wsdl settings
    for short_key_name in ('crab.wsdl',):
        key_name = "crabpy.%s" % short_key_name
        if key_name in settings:
            args[short_key_name] = settings.get(key_name)

    log.debug(settings)
    log.debug(args)
    return args


def _filter_settings(settings, prefix):
    """
    Filter all settings to only return settings that start with a certain
    prefix.

    :param dict settings: A settings dictionary.
    :param str prefix: A prefix.
    """
    ret = {}
    for skey in settings.keys():
        if skey.startswith(prefix):
            key = skey[len(prefix):]
            ret[key] = settings[skey]
    return ret


def _build_capakey(registry, settings):
    capakey = registry.queryUtility(ICapakey)
    if capakey is not None:
        return capakey
    if 'cache_config' in settings:
        cache_config = settings['cache_config']
        del settings['cache_config']
    else:
        cache_config = {}
    gateway = CapakeyRestGateway(cache_config=cache_config)

    registry.registerUtility(gateway, ICapakey)
    return registry.queryUtility(ICapakey)


def _build_crab(registry, settings):
    crab = registry.queryUtility(ICrab)
    if crab is not None:
        return crab
    if 'cache_config' in settings:
        cache_config = settings['cache_config']
        del settings['cache_config']
    else:
        cache_config = {}
    factory = crab_factory(**settings)
    gateway = CrabGateway(factory, cache_config=cache_config)

    registry.registerUtility(gateway, ICrab)
    return registry.queryUtility(ICrab)


def _build_adressenregister(registry, settings):
    adressenregister = registry.queryUtility(IAdressenregister)
    if adressenregister is not None:
        return adressenregister
    if 'cache_config' in settings:
        cache_config = settings['cache_config']
        del settings['cache_config']
    else:
        cache_config = None
    gateway = Gateway(
        client=AdressenRegisterClient(settings["base_url"], settings["api_key"]),
        cache_settings=cache_config
    )

    registry.registerUtility(gateway, IAdressenregister)
    return registry.queryUtility(IAdressenregister)


def get_capakey(registry):
    """
    Get the Capakey Gateway

    :rtype: :class:`crabpy.gateway.capakey.CapakeyRestGateway`
    """
    # argument might be a config or a request
    regis = getattr(registry, 'registry', None)
    if regis is None:
        regis = registry

    return regis.queryUtility(ICapakey)


def get_crab(registry):
    """
    Get the Crab Gateway

    :rtype: :class:`crabpy.gateway.crab.CrabGateway`
    # argument might be a config or a request
    """
    # argument might be a config or a request
    regis = getattr(registry, 'registry', None)
    if regis is None:
        regis = registry

    return regis.queryUtility(ICrab)


def get_adressenregister(registry):
    """
    Get the Adresssenregister Gateway

    :rtype: :class:`crabpy.gateway.adressenregister.Gateway`
    # argument might be a config or a request
    """
    # argument might be a config or a request
    regis = getattr(registry, 'registry', None)
    if regis is None:
        regis = registry

    return regis.queryUtility(IAdressenregister)


def _get_proxy_settings(settings):
    base_settings = {}
    http = settings.get('proxy.http', None)
    https = settings.get('proxy.https', None)
    if (http or https):
        base_settings["proxy"] = {}
        if "proxy.http" in settings:
            base_settings["proxy"]["http"] = settings["proxy.http"]
            log.info('HTTP proxy: %s' % base_settings["proxy"]["http"])
        if "proxy.https" in settings:
            base_settings["proxy"]["https"] = settings["proxy.https"]
            log.info('HTTPS proxy: %s' % base_settings["proxy"]["https"])
    return base_settings


def add_route(config, name, pattern, *args, **kwargs):
    """
    Adds a pyramid route to the config. All args and kwargs will be
    passed on to config.add_route.

    This exists so the default behaviour of including crabpy will still be to
    cache all crabpy routes.
    """
    config.add_route(name, pattern, *args, **kwargs)
    GENERATE_ETAG_ROUTE_NAMES.add(name)


def conditional_http_tween_factory(handler, registry):
    """
    Tween that adds ETag headers and tells Pyramid to enable 
    conditional responses where appropriate.
    """
    settings = registry.settings if hasattr(registry, 'settings') else {}
    if 'generate_etag_for.list' in settings:
        route_names = settings.get('generate_etag_for.list').split()
        GENERATE_ETAG_ROUTE_NAMES.update(route_names)

    def conditional_http_tween(request):
        response = handler(request)
        if request.matched_route is None:
            return response

        if request.matched_route.name in GENERATE_ETAG_ROUTE_NAMES:

            # If the Last-Modified header has been set, we want to enable the
            # conditional response processing.
            if response.last_modified is not None:
                response.conditional_response = True

            # We want to only enable the conditional machinery if either we
            # were given an explicit ETag header by the view or we have a
            # buffered response and can generate the ETag header ourself.
            if response.etag is not None:
                response.conditional_response = True
            elif (isinstance(response.app_iter, Sequence) and
                          len(response.app_iter) == 1) and response.body is not None:
                response.conditional_response = True
                response.md5_etag()

        return response

    return conditional_http_tween


def includeme(config):
    """
    Include `crabpy_pyramid` in this `Pyramid` application.

    :param pyramid.config.Configurator config: A Pyramid configurator.
    """

    settings = _parse_settings(config.registry.settings)
    base_settings = _get_proxy_settings(settings)

    # http caching tween
    if not settings.get('etag_tween_disabled', False):
        config.add_tween('crabpy_pyramid.conditional_http_tween_factory')

    # create cache
    root = settings.get('cache.file.root', '/tmp/dogpile_data')
    if not os.path.exists(root):
        os.makedirs(root)

    capakey_settings = dict(_filter_settings(settings, 'capakey.'),
                            **base_settings)
    if 'include' in capakey_settings:
        log.info("The 'capakey.include' setting is deprecated. Capakey will "
                 "always be included.")
    log.info('Adding CAPAKEY Gateway.')
    config.add_renderer('capakey_listjson', capakey_json_list_renderer)
    config.add_renderer('capakey_itemjson', capakey_json_item_renderer)
    _build_capakey(config.registry, capakey_settings)
    config.add_request_method(get_capakey, 'capakey_gateway')
    config.add_directive('get_capakey', get_capakey)
    config.include('crabpy_pyramid.routes.capakey')
    config.scan('crabpy_pyramid.views.capakey')

    crab_settings = dict(_filter_settings(settings, 'crab.'), **base_settings)
    if crab_settings['include']:
        log.info('Adding CRAB Gateway.')
        del crab_settings['include']
        config.add_renderer('crab_listjson', crab_json_list_renderer)
        config.add_renderer('crab_itemjson', crab_json_item_renderer)
        _build_crab(config.registry, crab_settings)
        config.add_directive('get_crab', get_crab)
        config.add_request_method(get_crab, 'crab_gateway')
        config.include('crabpy_pyramid.routes.crab')
        config.scan('crabpy_pyramid.views.crab')

    # adressenregister wordt afgekort tot adresreg
    adresreg_settings = dict(
        _filter_settings(settings, 'adressenregister.'),
        **base_settings
    )

    if adresreg_settings['include']:
        log.info('Adding adressen register Gateway.')
        del adresreg_settings['include']
        config.add_renderer('adresreg_listjson', adresreg_json_list_renderer)
        config.add_renderer('adresreg_itemjson', adresreg_json_item_renderer)
        _build_adressenregister(config.registry, adresreg_settings)
        config.add_directive('get_adressenregister', get_adressenregister)
        config.add_request_method(get_adressenregister, 'adressenregister_gateway')
        config.include('crabpy_pyramid.routes.adressenregister')
        config.scan('crabpy_pyramid.views.adressenregister')
        config.scan("crabpy_pyramid.views.exceptions")


def main(global_config, **settings):
    """
     This function returns a Pyramid WSGI application.
    """
    config = Configurator(settings=settings)

    includeme(config)
    return config.make_wsgi_app()



# -*- coding: utf-8 -*-
"""
Routes for the CRAB views.

.. versionadded:: 0.1.0
"""
import crabpy_pyramid


def includeme(config):
    crabpy_pyramid.add_route(config,
                             'list_gewesten',
                             '/crab/gewesten')
    crabpy_pyramid.add_route(config,
                             'get_gewest_by_id',
                             '/crab/gewesten/{gewest_id}')
    crabpy_pyramid.add_route(config,
                             'list_provincies',
                             '/crab/gewesten/{gewest_id}/provincies')
    crabpy_pyramid.add_route(config,
                             'get_provincie',
                             '/crab/provincies/{provincie_id}')
    crabpy_pyramid.add_route(config,
                             'list_gemeenten_by_provincie',
                             '/crab/provincies/{provincie_id}/gemeenten')
    crabpy_pyramid.add_route(config,
                             'list_gemeenten_crab',
                             '/crab/gewesten/{gewest_id}/gemeenten')
    crabpy_pyramid.add_route(config,
                             'get_gemeente_crab',
                             '/crab/gemeenten/{gemeente_id}')
    crabpy_pyramid.add_route(config,
                             'list_deelgemeenten',
                             '/crab/gewesten/{gewest_id}/deelgemeenten')
    crabpy_pyramid.add_route(config,
                             'list_deelgemeenten_by_gemeente',
                             '/crab/gemeenten/{gemeente_id}/deelgemeenten')
    crabpy_pyramid.add_route(config,
                             'get_deelgemeente_by_id',
                             '/crab/deelgemeenten/{deelgemeente_id}')
    crabpy_pyramid.add_route(config,
                             'list_straten',
                             '/crab/gemeenten/{gemeente_id}/straten')
    crabpy_pyramid.add_route(config,
                             'get_straat_by_id',
                             '/crab/straten/{straat_id}')
    crabpy_pyramid.add_route(config,
                             'get_wegobject',
                             '/crab/wegobjecten/{wegobject_id}')
    crabpy_pyramid.add_route(config,
                             'list_huisnummers',
                             '/crab/straten/{straat_id}/huisnummers')
    crabpy_pyramid.add_route(
        config,
        'get_huisnummer_by_straat_and_label',
        '/crab/straten/{straat_id}/huisnummers/{huisnummer_label}')
    crabpy_pyramid.add_route(config,
                             'get_huisnummer_by_id',
                             '/crab/huisnummers/{huisnummer_id}')
    crabpy_pyramid.add_route(config,
                             'list_percelen',
                             '/crab/huisnummers/{huisnummer_id}/percelen')
    crabpy_pyramid.add_route(config,
                             'get_perceel_by_id',
                             '/crab/percelen/{perceel_id1}/{perceel_id2}')
    crabpy_pyramid.add_route(
        config,
        'list_huisnummers_by_perceel',
        '/crab/percelen/{perceel_id1}/{perceel_id2}/huisnummers')
    crabpy_pyramid.add_route(config,
                             'list_gebouwen',
                             '/crab/huisnummers/{huisnummer_id}/gebouwen')
    crabpy_pyramid.add_route(config,
                             'get_gebouw_by_id',
                             '/crab/gebouwen/{gebouw_id}')
    crabpy_pyramid.add_route(config,
                             'list_subadressen',
                             '/crab/huisnummers/{huisnummer_id}/subadressen')
    crabpy_pyramid.add_route(config,
                             'get_subadres_by_id',
                             '/crab/subadressen/{subadres_id}')
    crabpy_pyramid.add_route(config,
                             'list_postkantons_by_gemeente',
                             '/crab/gemeenten/{gemeente_id}/postkantons')
    crabpy_pyramid.add_route(config,
                             'list_adresposities_by_huisnummer',
                             '/crab/huisnummers/{huisnummer_id}/adresposities')
    crabpy_pyramid.add_route(config,
                             'list_adresposities_by_subadres',
                             '/crab/subadressen/{subadres_id}/adresposities')
    crabpy_pyramid.add_route(config,
                             'get_adrespositie_by_id',
                             '/crab/adresposities/{adrespositie_id}')
    crabpy_pyramid.add_route(config,
                             'list_landen',
                             '/crab/landen')
    crabpy_pyramid.add_route(config,
                             'get_land_by_id',
                             '/crab/landen/{land_id}')
    crabpy_pyramid.add_route(config,
                             'get_postkanton_by_huisnummer',
                             '/crab/huisnummers/{huisnummer_id}/postkanton')



import pycountry
from crabpy.gateway import adressenregister
from pyramid.renderers import JSON

json_list_renderer = JSON()
json_item_renderer = JSON()


def list_gewesten_adapter(obj, request):
    """
    Adapter for rendering a list of
    :class:`crabpy.gateway.adressenregister.Gewest` to json.
    """
    return {
        "naam": obj.naam,
        "niscode": obj.niscode
    }


def list_provincie_adapter(obj, request):
    """
    Adapter for rendering a list of
    :class:`crabpy.gateway.adressenregister.Provincie` to json.
    """
    return {
        "niscode": obj.niscode,
        "naam": obj.naam,
        "gewest": {
            "niscode": obj.gewest_niscode,
        },
    }


def list_deelgemeente_adapter(obj, request):
    """
    Adapter for rendering a list of
    :class:`crabpy.gateway.adressenregister.Deelgemeente` to json.
    """
    return {
        "niscode": obj.id,
        "naam": obj.naam,
        "gemeente": {
            "niscode": obj.gemeente_niscode
        },
    }


def list_gemeente_adapter(obj, request):
    """
    Adapter for rendering a list of
    :class:`crabpy.gateway.adressenregister.Gemeenten` to json.
    """
    return {
        "niscode": obj.niscode,
        "naam": obj.naam(),
        "provincie": {
            "niscode": obj.provincie_niscode
        }
    }


def list_straten_adapter(obj, request):
    """
    Adapter for rendering a list of
    :class:`crabpy.gateway.adresregister.Straat` to json.
    """
    return {
        "id": obj.id,
        "naam": obj.naam,
        "status": obj.status,
        "uri": obj.uri
    }


def list_adressen_adapter(obj, request):
    """
    Adapter for rendering a list of
    :class:`crabpy.gateway.adresregister.Adres` to json.
    """
    return {
        "id": obj.id,
        "uri": obj.uri,
        "label": obj.label,
        "huisnummer": obj.huisnummer,
        "busnummer": obj.busnummer,
        "status": obj.status,
    }


def list_percelen_adapter(obj, request):
    """
    Adapter for rendering a list of
    :class:`crabpy.gateway.adressenregister.Perceel` to json.
    """
    return {
        "id": obj.id,
        "uri": obj.uri,
        "status": obj.status
    }


def list_postinfo_adapter(obj, request):
    """
    Adapter for rendering a list of
    :class:`crabpy.gateway.adressenregister.Postinfo` to json.
    """
    return {
        "postcode": obj.id,
        "uri": obj.uri,
        "status": obj.status,
        "namen": obj.namen(),
    }


def list_landen_adapter(obj, request):
    """
    Adapter for rendering a list of landen to json.
    """
    return {
        "id": obj.alpha_2,
        "naam": _(obj.name)
    }


json_list_renderer.add_adapter(adressenregister.Gewest, list_gewesten_adapter)
json_list_renderer.add_adapter(adressenregister.Provincie, list_provincie_adapter)
json_list_renderer.add_adapter(adressenregister.Deelgemeente, list_deelgemeente_adapter)
json_list_renderer.add_adapter(adressenregister.Gemeente, list_gemeente_adapter)
json_list_renderer.add_adapter(adressenregister.Straat, list_straten_adapter)
json_list_renderer.add_adapter(adressenregister.Adres, list_adressen_adapter)
json_list_renderer.add_adapter(adressenregister.Perceel, list_percelen_adapter)
json_list_renderer.add_adapter(adressenregister.Postinfo, list_postinfo_adapter)
json_list_renderer.add_adapter(pycountry.db.Data, list_landen_adapter)


def item_gewest_adapter(obj, request):
    """
    Adapter for rendering an object of
    :class:`crabpy.gateway.adressenregister.Gewest` to json.
    """
    return {
        "niscode": obj.niscode,
        "naam": obj.naam,
        "centroid": obj.centroid,
        "bounding_box": obj.bounding_box,
    }


def item_provincie_adapter(obj, request):
    """
    Adapter for rendering a object of
    :class:`crabpy.gateway.adressenregister.Provincie` to json.
    """
    return {
        "niscode": obj.niscode,
        "naam": obj.naam,
        "gewest": {
            "niscode": obj.gewest_niscode,
        },
    }


def item_gemeente_adapter(obj, request):
    """
    Adapter for rendering an object of
    :class:`crabpy.gateway.adressenregister.Gemeente` to json.
    """
    return {
        "niscode": obj.niscode,
        "naam": obj.naam(),
        "provincie": {"niscode": obj.provincie_niscode},
        "gewest": {"niscode": obj.gewest.niscode}
    }


def item_deelgemeente_adapter(obj, request):
    """
    Adapter for rendering a object of
    :class:`crabpy.gateway.adressenregister.Deelgemeente` to json.
    """
    return {
        "niscode": obj.id,
        "naam": obj.naam,
        "gemeente": {
            "niscode": obj.gemeente.niscode,
            "naam": obj.gemeente.naam()
        },
    }


def item_straat_adapter(obj, request):
    """
    Adapter for rendering an object of
    :class:`crabpy.gateway.adressenregister.Straat` to json.
    """
    return {
        "id": obj.id,
        "naam": obj.naam(),
        "status": obj.status,
        "uri": obj.uri
    }


def item_adres_adapter(obj, request):
    """
    Adapter for rendering a list of
    :class:`crabpy.gateway.adresregister.Adres` to json.
    """
    return {
        "id": obj.id,
        "uri": obj.uri,
        "label": obj.label,
        "huisnummer": obj.huisnummer,
        "busnummer": obj.busnummer,
        "status": obj.status,
    }


def item_perceel_adapter(obj, request):
    """
    Adapter for rendering an object of
    :class:`crabpy.gateway.adressenregister.Perceel` to json.
    """
    return {
        "id": obj.id,
        "uri": obj.uri,
        "status": obj.status,
        "adressen": [{"id": adres.id} for adres in obj.adressen],
    }


def item_land_adapter(obj, request):
    """
    Adapter for rendering an item of
    :class: `pycountry.db.Data` to json.
    """
    return {
        "id": obj.alpha_2,
        "alpha2": obj.alpha_2,
        "alpha3": obj.alpha_3,
        "naam": obj.name,
    }


def list_postinfo_adapter(obj, request):
    """
    Adapter for rendering a list of
    :class:`crabpy.gateway.adressenregister.Postinfo` to json.
    """
    return {
        "postcode": obj.id,
        "uri": obj.uri,
        "status": obj.status,
        "namen": obj.namen(),
    }


json_item_renderer.add_adapter(adressenregister.Gewest, item_gewest_adapter)
json_item_renderer.add_adapter(adressenregister.Provincie, item_provincie_adapter)
json_item_renderer.add_adapter(adressenregister.Deelgemeente, item_deelgemeente_adapter)
json_item_renderer.add_adapter(adressenregister.Gemeente, item_gemeente_adapter)
json_item_renderer.add_adapter(adressenregister.Straat, item_straat_adapter)
json_item_renderer.add_adapter(adressenregister.Perceel, item_perceel_adapter)
json_item_renderer.add_adapter(adressenregister.Adres, item_adres_adapter)
json_item_renderer.add_adapter(adressenregister.Postinfo, list_postinfo_adapter)
json_item_renderer.add_adapter(pycountry.db.Data, item_land_adapter)



import logging
import re

import pycountry
from crabpy.client import AdressenRegisterClientException
from pyramid.httpexceptions import HTTPBadRequest
from pyramid.httpexceptions import HTTPNotFound
from pyramid.view import view_config

from crabpy_pyramid.utils import range_return
from crabpy_pyramid.utils import set_http_caching

log = logging.getLogger(__name__)


def handle_gateway_response(gateway_method, *args, **kwargs):
    try:
        result = gateway_method(*args, **kwargs)
        if not result:
            raise HTTPNotFound()
        return result
    except AdressenRegisterClientException as ae:
        cause = ae.__cause__
        if hasattr(cause, "response"):
            status_code = cause.response.status_code
            if status_code == 404:
                raise HTTPNotFound()
            if status_code == 400:
                detail = getattr(cause.response, "text", "")
                raise HTTPBadRequest(detail=detail)
        raise ae


@view_config(
    route_name="adressenregister_list_gewesten",
    renderer="adresreg_listjson",
    accept="application/json",
)
def list_gewesten(request):
    request = set_http_caching(request, "adressenregister", "long")
    Gateway = request.adressenregister_gateway()
    gewesten = Gateway.list_gewesten()
    return range_return(request, gewesten)


@view_config(
    route_name="adressenregister_get_gewest_by_niscode",
    renderer="adresreg_itemjson",
    accept="application/json",
)
def get_gewest_by_niscode(request):
    request = set_http_caching(request, "adressenregister", "long")
    Gateway = request.adressenregister_gateway()
    gewest_niscode = request.matchdict.get("gewest_niscode")
    return handle_gateway_response(Gateway.get_gewest_by_niscode, gewest_niscode)


@view_config(
    route_name="adressenregister_list_provincies",
    renderer="adresreg_listjson",
    accept="application/json",
)
def list_provincies(request):
    request = set_http_caching(request, "adressenregister", "permanent")
    Gateway = request.adressenregister_gateway()
    gewest_niscode = request.matchdict.get("gewest_niscode")
    provincies = handle_gateway_response(Gateway.list_provincies, gewest_niscode)
    return range_return(request, provincies)


@view_config(
    route_name="adressenregister_get_provincie",
    renderer="adresreg_itemjson",
    accept="application/json",
)
def get_provincie(request):
    request = set_http_caching(request, "adressenregister", "permanent")
    Gateway = request.adressenregister_gateway()
    provincie_niscode = request.matchdict.get("provincie_niscode")

    return handle_gateway_response(Gateway.get_provincie_by_niscode, provincie_niscode)


@view_config(
    route_name="adressenregister_list_deelgemeenten",
    renderer="adresreg_listjson",
    accept="application/json",
)
def list_deelgemeenten(request):
    request = set_http_caching(request, "adressenregister", "permanent")
    Gateway = request.adressenregister_gateway()
    gewest_niscode = request.matchdict.get("gewest_niscode")
    if gewest_niscode != "2000":
        raise HTTPNotFound()
    deelgemeenten = handle_gateway_response(Gateway.list_deelgemeenten, gewest_niscode)

    return range_return(request, deelgemeenten)


@view_config(
    route_name="adressenregister_list_deelgemeenten_by_gemeente",
    renderer="adresreg_listjson",
    accept="application/json",
)
def list_deelgemeenten_by_gemeente(request):
    request = set_http_caching(request, "adressenregister", "permanent")
    Gateway = request.adressenregister_gateway()
    gemeente_id = request.matchdict.get("niscode")
    gemeente = handle_gateway_response(Gateway.get_gemeente_by_niscode, gemeente_id)
    deelgemeenten = Gateway.list_deelgemeenten_by_gemeente(gemeente)

    return range_return(request, deelgemeenten)


@view_config(
    route_name="adressenregister_get_deelgemeente_by_id",
    renderer="adresreg_itemjson",
    accept="application/json",
)
def get_deelgemeente_by_id(request):
    request = set_http_caching(request, "adressenregister", "permanent")
    Gateway = request.adressenregister_gateway()
    deelgemeente_niscode = request.matchdict.get("deelgemeente_niscode")

    return handle_gateway_response(Gateway.get_deelgemeente_by_id, deelgemeente_niscode)


@view_config(
    route_name="adressenregister_list_gemeenten_by_provincie",
    renderer="adresreg_listjson",
    accept="application/json",
)
def list_gemeenten_by_provincie(request):
    request = set_http_caching(request, "adressenregister", "long")
    Gateway = request.adressenregister_gateway()
    provincie_id = request.matchdict.get("provincie_niscode")
    gemeenten = handle_gateway_response(
        Gateway.list_gemeenten_by_provincie, provincie_id
    )

    return range_return(request, gemeenten)


@view_config(
    route_name="list_gemeenten_adressenregister",
    renderer="adresreg_listjson",
    accept="application/json",
)
def list_gemeenten_adressenregister(request):
    request = set_http_caching(request, "adressenregister", "long")
    Gateway = request.adressenregister_gateway()
    gewest_niscode = request.matchdict.get("gewest_niscode")
    gemeenten = handle_gateway_response(Gateway.list_gemeenten, gewest_niscode)

    return range_return(request, gemeenten)


@view_config(
    route_name="get_gemeente_adressenregister",
    renderer="adresreg_itemjson",
    accept="application/json",
)
def get_gemeente_adressenregister(request):
    request = set_http_caching(request, "adressenregister", "long")
    Gateway = request.adressenregister_gateway()
    gemeente_niscode = request.matchdict.get("niscode")

    return handle_gateway_response(Gateway.get_gemeente_by_niscode, gemeente_niscode)


@view_config(
    route_name="adressenregister_list_straten",
    renderer="adresreg_listjson",
    accept="application/json",
)
def list_straten(request):
    request = set_http_caching(request, "adressenregister", "short")
    Gateway = request.adressenregister_gateway()
    gemeente_niscode = request.matchdict.get("niscode")
    straten = handle_gateway_response(Gateway.list_straten, gemeente_niscode)

    return range_return(request, straten)


@view_config(
    route_name="adressenregister_get_straat_by_id",
    renderer="adresreg_itemjson",
    accept="application/json",
)
def get_straat_by_id(request):
    request = set_http_caching(request, "adressenregister", "short")
    Gateway = request.adressenregister_gateway()
    straat_id = request.matchdict.get("straat_id")

    return handle_gateway_response(Gateway.get_straat_by_id, straat_id)


@view_config(
    route_name="adressenregister_list_adressen",
    renderer="adresreg_listjson",
    accept="application/json",
)
def list_adressen_by_straat(request):
    request = set_http_caching(request, "adressenregister", "short")
    Gateway = request.adressenregister_gateway()
    straat_id = request.matchdict.get("straat_id")
    straten = handle_gateway_response(
        Gateway.list_adressen_with_params, straatnaamObjectId=straat_id
    )

    return range_return(request, straten)


@view_config(
    route_name="adressenregister_get_adres_by_straat_and_huisnummer",
    renderer="adresreg_listjson",
    accept="application/json",
)
def adressenregister_get_adres_by_straat_huisnummer(request):
    request = set_http_caching(request, "adressenregister", "short")
    Gateway = request.adressenregister_gateway()
    straat_id = request.matchdict.get("straat_id")
    huisnummer = request.matchdict.get("huisnummer")
    adressen = handle_gateway_response(
        Gateway.list_adressen_with_params,
        straatnaamObjectId=straat_id,
        huisnummer=huisnummer,
    )

    return range_return(request, adressen)


@view_config(
    route_name="adressenregister_get_adres_by_straat_and_huisnummer_and_busnummer",
    renderer="adresreg_listjson",
    accept="application/json",
)
def adressenregister_get_adres_by_straat_huisnummer_busnummer(request):
    request = set_http_caching(request, "adressenregister", "short")
    Gateway = request.adressenregister_gateway()
    straat_id = request.matchdict.get("straat_id")
    huisnummer = request.matchdict.get("huisnummer")
    busnummer = request.matchdict.get("busnummer")
    adressen = handle_gateway_response(
        Gateway.list_adressen_with_params,
        straatnaamObjectId=straat_id,
        huisnummer=huisnummer,
        busnummer=busnummer,
    )

    return range_return(request, adressen)


@view_config(
    route_name="adressenregister_get_adres_by_id",
    renderer="adresreg_itemjson",
    accept="application/json",
)
def adressenregister_get_adres_by_id(request):
    request = set_http_caching(request, "adressenregister", "long")
    Gateway = request.adressenregister_gateway()
    adres_id = request.matchdict.get("adres_id")

    return handle_gateway_response(Gateway.get_adres_by_id, adres_id)


@view_config(
    route_name="adressenregister_list_percelen_by_adres",
    renderer="adresreg_listjson",
    accept="application/json",
)
def adressenregister_list_percelen_by_adres(request):
    request = set_http_caching(request, "adressenregister", "short")
    Gateway = request.adressenregister_gateway()
    adres_id = request.matchdict.get("adres_id")
    adressen = handle_gateway_response(
        Gateway.list_percelen_with_params, adresObjectId=adres_id
    )

    return range_return(request, adressen)


@view_config(
    route_name="adressenregister_get_perceel_by_id",
    renderer="adresreg_itemjson",
    accept="application/json",
)
def adressenregister_get_perceel_by_id(request):
    request = set_http_caching(request, "adressenregister", "short")
    Gateway = request.adressenregister_gateway()
    perceel_id = request.matchdict.get("perceel_id")
    return handle_gateway_response(Gateway.get_perceel_by_id, perceel_id=perceel_id)


@view_config(
    route_name="adressenregister_get_perceel_by_id_parts",
    renderer="adresreg_itemjson",
    accept="application/json",
)
def adressenregister_get_perceel_by_id_parts(request):
    request = set_http_caching(request, "adressenregister", "short")
    Gateway = request.adressenregister_gateway()
    perceel_id = (
        f'{request.matchdict.get("perceel_id_part1")}'
        f'-{request.matchdict.get("perceel_id_part2")}'
    )
    return handle_gateway_response(Gateway.get_perceel_by_id, perceel_id=perceel_id)


@view_config(
    route_name="adressenregister_list_postinfo_by_gemeente",
    renderer="adresreg_listjson",
    accept="application/json",
)
def adressenregister_list_postinfo_by_gemeente(request):
    request = set_http_caching(request, "adressenregister", "long")
    Gateway = request.adressenregister_gateway()
    gemeente_param = request.matchdict.get("gemeente_naam_niscode")
    niscode_pattern = re.compile(r"^\d{5}$")
    if niscode_pattern.match(gemeente_param):
        gemeente = handle_gateway_response(
            Gateway.get_gemeente_by_niscode, gemeente_param
        )
        gemeente_param = gemeente.naam()
    adressen = handle_gateway_response(
        Gateway.get_postinfo_by_gemeentenaam, gemeente_param
    )

    return range_return(request, adressen)


@view_config(
    route_name="adressenregister_get_postinfo_by_postcode",
    renderer="adresreg_itemjson",
    accept="application/json",
)
def adressenregister_get_postinfo_by_postcode(request):
    request = set_http_caching(request, "adressenregister", "long")
    Gateway = request.adressenregister_gateway()
    postcode = request.matchdict.get("postcode")

    return handle_gateway_response(Gateway.get_postinfo_by_id, postcode)


@view_config(
    route_name="adressenregister_list_landen",
    renderer="adresreg_listjson",
    accept="application/json",
)
def list_landen(request):
    set_http_caching(request, "adressenregister", "permanent")
    return list(pycountry.countries)


@view_config(
    route_name="adressenregister_get_land_by_id",
    renderer="adresreg_itemjson",
    accept="application/json",
)
def get_land_by_id(request):
    request = set_http_caching(request, "adressenregister", "permanent")
    land_id = request.matchdict.get("land_id")
    land = pycountry.countries.get(alpha_2=land_id)
    if land is None:
        raise HTTPNotFound()
    return land



# -*- coding: utf-8 -*-
"""
Adapters to help render :mod:`crabpy.gateway.crab` objects to json.

.. versionadded:: 0.1.0
"""
import gettext

import pycountry
from crabpy.gateway import crab
from pyramid.renderers import JSON

json_list_renderer = JSON()
json_item_renderer = JSON()

nederlands = gettext.translation('iso3166', pycountry.LOCALES_DIR, languages=['nl'])
nederlands.install()


def list_gewesten_adapter(obj, request):
    """
    Adapter for rendering a list of
    :class:`crabpy.gateway.crab.Gewest` to json.
    """
    return {
        'id': obj.id,
        'naam': obj.naam
    }


def list_provincie_adapter(obj, request):
    """
    Adapter for rendering a list of
    :class:`crabpy.gateway.crab.Provincie` to json.
    """
    return {
        'niscode': obj.niscode,
        'naam': obj.naam,
        'gewest': {
            'id': obj.gewest.id,
            'naam': obj.gewest.naam
        }
    }


def list_gemeente_adapter(obj, request):
    """
    Adapter for rendering a list of
    :class:`crabpy.gateway.crab.Gemeenten` to json.
    """
    return {
        'id': obj.id,
        'niscode': obj.niscode,
        'naam': obj.naam
    }


def list_deelgemeente_adapter(obj, request):
    '''
    Adapter for rendering a list of
    :class:`crabpy.gateway.crab.Deelgemeente` to json.
    '''
    return {
        'id': obj.id,
        'naam': obj.naam
    }


def list_straten_adapter(obj, request):
    """
    Adapter for rendering a list of
    :class:`crabpy.gateway.crab.Straat` to json.
    """
    naam = obj.label
    for name, language in obj.namen:
        if language == 'nl' and name:
            naam = name
            break
    return {
        'id': obj.id,
        'label': obj.label,
        'naam': naam,
        'status': {
            'id': obj.status.id,
            'naam': obj.status.naam,
            'definitie': obj.status.definitie
        },
    }


def list_huisnummers_adapter(obj, request):
    """
    Adapter for rendering a list of
    :class:`crabpy.gateway.crab.Huisnummer` to json.
    """
    return {
        'id': obj.id,
        'status': {
            'id': obj.status.id,
            'naam': obj.status.naam,
            'definitie': obj.status.definitie
        },
        'label': obj.huisnummer
    }


def list_percelen_adapter(obj, request):
    """
    Adapter for rendering a list of
    :class:`crabpy.gateway.crab.Perceel` to json.
    """
    return {
        'id': obj.id
    }


def list_gebouwen_adapter(obj, request):
    """
    Adapter for rendering a list of
    :class:`crabpy.gateway.crab.Gebouw` to json.
    """
    return {
        'id': obj.id,
        'aard': {
            'id': obj.aard.id,
            'naam': obj.aard.naam,
            'definitie': obj.aard.definitie
        },
        'status': {
            'id': obj.status.id,
            'naam': obj.status.naam,
            'definitie': obj.status.definitie
        }
    }


def list_subadres_adapter(obj, request):
    """
    Adapter for rendering a list of
    :class:`crabpy.gateway.crab.Subadres` to json.
    """
    return {
        'id': obj.id,
        'subadres': obj.subadres,
        'status': {
            'id': obj.status.id,
            'naam': obj.status.naam,
            'definitie': obj.status.definitie
        }
    }


def list_postkantons_adapter(obj, request):
    """
    Adapter for rendering a list of
    :class:`crabpy.gateway.crab.Postkanton` to json.
    """
    return {
        'id': obj.id
    }


def list_adresposities_adapter(obj, request):
    """
    Adapter for rendering a list of
    :class:`crabpy.gateway.crab.Adrespositie` to json.
    """
    return {
        'id': obj.id,
        'herkomst': {
            'id': obj.herkomst.id,
            'naam': obj.herkomst.naam,
            'definitie': obj.herkomst.definitie
        }
    }


def list_landen_adapter(obj, request):
    """
    Adapter for rendering a list of landen to json.
    """
    return {
        'id': obj.alpha_2,
        'naam': _(obj.name)
    }


json_list_renderer.add_adapter(crab.Gewest, list_gewesten_adapter)
json_list_renderer.add_adapter(crab.Provincie, list_provincie_adapter)
json_list_renderer.add_adapter(crab.Gemeente, list_gemeente_adapter)
json_list_renderer.add_adapter(crab.Deelgemeente, list_deelgemeente_adapter)
json_list_renderer.add_adapter(crab.Straat, list_straten_adapter)
json_list_renderer.add_adapter(crab.Huisnummer, list_huisnummers_adapter)
json_list_renderer.add_adapter(crab.Perceel, list_percelen_adapter)
json_list_renderer.add_adapter(crab.Gebouw, list_gebouwen_adapter)
json_list_renderer.add_adapter(crab.Subadres, list_subadres_adapter)
json_list_renderer.add_adapter(crab.Postkanton, list_postkantons_adapter)
json_list_renderer.add_adapter(crab.Adrespositie, list_adresposities_adapter)
json_list_renderer.add_adapter(pycountry.db.Data, list_landen_adapter)


def item_gewest_adapter(obj, request):
    """
    Adapter for rendering an object of
    :class:`crabpy.gateway.crab.Gewest` to json.
    """
    return {
        'id': obj.id,
        'namen': obj._namen,
        'centroid': obj.centroid,
        'bounding_box': obj.bounding_box
    }


def item_provincie_adapter(obj, request):
    """
    Adapter for rendering a object of
    :class:`crabpy.gateway.crab.Provincie` to json.
    """
    return {
        'niscode': obj.niscode,
        'naam': obj.naam,
        'gewest': {
            'id': obj.gewest.id,
            'naam': obj.gewest.naam
        }
    }


def item_gemeente_adapter(obj, request):
    """
    Adapter for rendering an object of
    :class:`crabpy.gateway.crab.Gemeente` to json.
    """
    return {
        'id': obj.id,
        'niscode': obj.niscode,
        'naam': obj.naam,
        'centroid': obj.centroid,
        'bounding_box': obj.bounding_box,
        'metadata': {
            'begin_tijd': obj.metadata.begin_tijd,
            'begin_datum': obj.metadata.begin_datum,
            'begin_bewerking': {
                'id': obj.metadata.begin_bewerking.id,
                'naam': obj.metadata.begin_bewerking.naam,
                'definitie': obj.metadata.begin_bewerking.definitie
            },
            'begin_organisatie': {
                'id': obj.metadata.begin_organisatie.id,
                'naam': obj.metadata.begin_organisatie.naam,
                'definitie': obj.metadata.begin_organisatie.definitie
            }
        }
    }


def item_deelgemeente_adapter(obj, request):
    """
    Adapter for rendering a object of
    :class:`crabpy.gateway.crab.Deelgemeente` to json.
    """
    return {
        'id': obj.id,
        'naam': obj.naam,
        'gemeente': {
            'id': obj.gemeente.id,
            'naam': obj.gemeente.naam
        }
    }


def item_straat_adapter(obj, request):
    """
    Adapter for rendering an object of
    :class:`crabpy.gateway.crab.Straat` to json.
    """
    return {
        'id': obj.id,
        'label': obj.label,
        'namen': obj.namen,
        'status': {
            'id': obj.status.id,
            'naam': obj.status.naam,
            'definitie': obj.status.definitie
        },
        'taal': {
            'id': obj.taal.id,
            'naam': obj.taal.naam,
            'definitie': obj.taal.definitie
        },
        'metadata': {
            'begin_tijd': obj.metadata.begin_tijd,
            'begin_datum': obj.metadata.begin_datum,
            'begin_bewerking': {
                'id': obj.metadata.begin_bewerking.id,
                'naam': obj.metadata.begin_bewerking.naam,
                'definitie': obj.metadata.begin_bewerking.definitie
            },
            'begin_organisatie': {
                'id': obj.metadata.begin_organisatie.id,
                'naam': obj.metadata.begin_organisatie.naam,
                'definitie': obj.metadata.begin_organisatie.definitie
            }
        },
        'bounding_box': obj.bounding_box
    }


def item_huisnummer_adapter(obj, request):
    """
    Adapter for rendering an object of
    :class:`crabpy.gateway.crab.Huisnummer` to json.
    """
    return {
        'id': obj.id,
        'huisnummer': obj.huisnummer,
        'postadres': obj.postadres,
        'status': {
            'id': obj.status.id,
            'naam': obj.status.naam,
            'definitie': obj.status.definitie
        },
        'metadata': {
            'begin_tijd': obj.metadata.begin_tijd,
            'begin_datum': obj.metadata.begin_datum,
            'begin_bewerking': {
                'id': obj.metadata.begin_bewerking.id,
                'naam': obj.metadata.begin_bewerking.naam,
                'definitie': obj.metadata.begin_bewerking.definitie
            },
            'begin_organisatie': {
                'id': obj.metadata.begin_organisatie.id,
                'naam': obj.metadata.begin_organisatie.naam,
                'definitie': obj.metadata.begin_organisatie.definitie
            }
        },
        'bounding_box': obj.bounding_box
    }


def item_perceel_crab_adapter(obj, request):
    """
    Adapter for rendering an object of
    :class:`crabpy.gateway.crab.Perceel` to json.
    """
    return {
        'id': obj.id,
        'centroid': obj.centroid,
        'postadressen': obj.postadressen,
        'metadata': {
            'begin_tijd': obj.metadata.begin_tijd,
            'begin_datum': obj.metadata.begin_datum,
            'begin_bewerking': {
                'id': obj.metadata.begin_bewerking.id,
                'naam': obj.metadata.begin_bewerking.naam,
                'definitie': obj.metadata.begin_bewerking.definitie
            },
            'begin_organisatie': {
                'id': obj.metadata.begin_organisatie.id,
                'naam': obj.metadata.begin_organisatie.naam,
                'definitie': obj.metadata.begin_organisatie.definitie
            }
        }
    }


def item_gebouw_adapter(obj, request):
    """
    Adapter for rendering an object of
    :class:`crabpy.gateway.crab.Gebouw` to json.
    """
    return {
        'id': obj.id,
        'aard': {
            'id': obj.aard.id,
            'naam': obj.aard.naam,
            'definitie': obj.aard.definitie
        },
        'status': {
            'id': obj.status.id,
            'naam': obj.status.naam,
            'definitie': obj.status.definitie
        },
        'geometriemethode': {
            'id': obj.methode.id,
            'naam': obj.methode.naam,
            'definitie': obj.methode.definitie
        },
        'geometrie': obj.geometrie,
        'metadata': {
            'begin_tijd': obj.metadata.begin_tijd,
            'begin_datum': obj.metadata.begin_datum,
            'begin_bewerking': {
                'id': obj.metadata.begin_bewerking.id,
                'naam': obj.metadata.begin_bewerking.naam,
                'definitie': obj.metadata.begin_bewerking.definitie
            },
            'begin_organisatie': {
                'id': obj.metadata.begin_organisatie.id,
                'naam': obj.metadata.begin_organisatie.naam,
                'definitie': obj.metadata.begin_organisatie.definitie
            }
        }
    }


def item_wegobject_adapter(obj, request):
    """
    Adapter for rendering a list of
    :class:`crabpy.gateway.Wegobject` to json.
    """
    return {
        'id': obj.id,
        'aard': {
            'id': obj.aard.id,
            'naam': obj.aard.naam,
            'definitie': obj.aard.definitie
        },
        'centroid': obj.centroid,
        'bounding_box': obj.bounding_box,
        'metadata': {
            'begin_tijd': obj.metadata.begin_tijd,
            'begin_datum': obj.metadata.begin_datum,
            'begin_bewerking': {
                'id': obj.metadata.begin_bewerking.id,
                'naam': obj.metadata.begin_bewerking.naam,
                'definitie': obj.metadata.begin_bewerking.definitie
            },
            'begin_organisatie': {
                'id': obj.metadata.begin_organisatie.id,
                'naam': obj.metadata.begin_organisatie.naam,
                'definitie': obj.metadata.begin_organisatie.definitie
            }
        }
    }


def item_subadres_adapter(obj, request):
    """
    Adapter for rendering an item of
    :class:`crabpy.gateway.Subadres` to json.
    """
    return {
        'id': obj.id,
        'subadres': obj.subadres,
        'postadres': obj.postadres,
        'status': {
            'id': obj.status.id,
            'naam': obj.status.naam,
            'definitie': obj.status.definitie
        },
        'aard': {
            'id': obj.aard.id,
            'naam': obj.aard.naam,
            'definitie': obj.aard.definitie
        },
        'metadata': {
            'begin_tijd': obj.metadata.begin_tijd,
            'begin_datum': obj.metadata.begin_datum,
            'begin_bewerking': {
                'id': obj.metadata.begin_bewerking.id,
                'naam': obj.metadata.begin_bewerking.naam,
                'definitie': obj.metadata.begin_bewerking.definitie
            },
            'begin_organisatie': {
                'id': obj.metadata.begin_organisatie.id,
                'naam': obj.metadata.begin_organisatie.naam,
                'definitie': obj.metadata.begin_organisatie.definitie
            }
        }
    }


def item_adrespositie_adapter(obj, request):
    """
    Adapter for rendering an item of
    :class:`crabpy.gateway.Adrespositie` to json.
    """
    return {
        'id': obj.id,
        'herkomst': {
            'id': obj.herkomst.id,
            'naam': obj.herkomst.naam,
            'definitie': obj.herkomst.definitie
        },
        'geometrie': obj.geometrie,
        'aard': {
            'id': obj.aard.id,
            'naam': obj.aard.naam,
            'definitie': obj.aard.definitie
        },
        'metadata': {
            'begin_tijd': obj.metadata.begin_tijd,
            'begin_datum': obj.metadata.begin_datum,
            'begin_bewerking': {
                'id': obj.metadata.begin_bewerking.id,
                'naam': obj.metadata.begin_bewerking.naam,
                'definitie': obj.metadata.begin_bewerking.definitie
            },
            'begin_organisatie': {
                'id': obj.metadata.begin_organisatie.id,
                'naam': obj.metadata.begin_organisatie.naam,
                'definitie': obj.metadata.begin_organisatie.definitie
            }
        }
    }


def item_land_adapter(obj, request):
    """
    Adapter for rendering an item of
    :class: `pycountry.db.Data` to json.
    """
    return {
        'id': obj.alpha_2,
        'alpha2': obj.alpha_2,
        'alpha3': obj.alpha_3,
        'naam': _(obj.name)
    }


def item_postkanton_adapter(obj, request):
    """
    Adapter for rendering an item of
    :class: `pycountry.db.Data` to json.
    """
    return {
        'postcode': obj.id,
    }


json_item_renderer.add_adapter(crab.Gewest, item_gewest_adapter)
json_item_renderer.add_adapter(crab.Provincie, item_provincie_adapter)
json_item_renderer.add_adapter(crab.Gemeente, item_gemeente_adapter)
json_item_renderer.add_adapter(crab.Deelgemeente, item_deelgemeente_adapter)
json_item_renderer.add_adapter(crab.Straat, item_straat_adapter)
json_item_renderer.add_adapter(crab.Huisnummer, item_huisnummer_adapter)
json_item_renderer.add_adapter(crab.Perceel, item_perceel_crab_adapter)
json_item_renderer.add_adapter(crab.Gebouw, item_gebouw_adapter)
json_item_renderer.add_adapter(crab.Wegobject, item_wegobject_adapter)
json_item_renderer.add_adapter(crab.Subadres, item_subadres_adapter)
json_item_renderer.add_adapter(crab.Adrespositie, item_adrespositie_adapter)
json_item_renderer.add_adapter(pycountry.db.Data, item_land_adapter)
json_item_renderer.add_adapter(crab.Postkanton, item_postkanton_adapter)
